import * as pathToRegexp from 'path-to-regexp';
export declare const parseQuery: (query: string) => any;
export declare const stringifyQuery: (query: Object) => string;
export interface Object {
    [index: string]: any;
    error?: any;
    status?: any;
}
export interface RootRoute {
    path?: string | RegExp;
    action?: Action | Middleware;
    childs: Array<RawRoute>;
    status?: number;
    to?: never;
}
export interface RawRoute {
    path: string | RegExp;
    action?: Action | Middleware;
    childs?: Array<RawRoute>;
    to?: string;
    status?: number;
    [index: string]: any;
}
export interface Route {
    path: string | RegExp;
    action?: Action | Middleware;
    pattern: RegExp;
    keys?: Array<pathToRegexp.Key>;
    status?: number;
    to?: string;
    [index: string]: any;
}
export interface Action {
    (options: ActionOptions): any;
}
export interface Middleware {
    (next: Action | Middleware, options: ActionOptions): any;
}
export interface ActionOptions {
    path: string;
    location: Location;
    route: Route;
    status: number;
    params: Object;
    redirect: string | null;
    ctx: Context;
}
export interface Location {
    pathname: string;
    search: string;
    hash: string;
    query: Object;
}
export interface RouterResult {
    path: string;
    location: Location;
    route: Route;
    status: number;
    params: Object;
    redirect: string | null;
    result: any;
    ctx: Context;
    error: null | RouterError;
}
export declare class RouterError {
    message: string;
    status: number;
    constructor(message?: string, status?: number);
}
export declare class DynamicRedirect {
    path: string;
    status: number;
    constructor(path: string, status?: number);
}
export declare class Context {
    private keys;
    constructor();
    set(key: string, value: any): void;
    get(key: string): any;
    toObject(): {
        [index: string]: any;
        error?: any;
        status?: any;
    };
}
export declare class Transition {
    isCancelled: boolean;
    cancel(): void;
    runOrResolve(path: string, ctx: Context, isHooks: boolean, router: Router): Promise<Object>;
}
export declare class Router {
    routes: Array<Route>;
    private hooks;
    private complete;
    helpers: any;
    isRunning: boolean;
    currentTransition: Transition;
    constructor({routes, hooks, helpers}: {
        routes: RootRoute | Array<RawRoute>;
        hooks?: Object;
        helpers?: any;
    });
    private walk(route, walkPath?);
    private matchRoute(path);
    private checkIsRunning(path, ctx, isHook);
    handleError(params: Object, transition: Transition, isHooks: boolean): Promise<{
        [index: string]: any;
        error?: any;
        status?: any;
    }>;
    runHooks(hook: string, transition: Transition, options?: Object, isHooks?: boolean): Promise<{
        [index: string]: any;
        error?: any;
        status?: any;
    } | null>;
    match({path, ctx}: {
        path: string;
        ctx: Context;
    }): Promise<{
        route: any;
        status: any;
        redirect: any;
        error: any;
        params: null;
    } | {
        route: any;
        status: any;
        redirect: any;
        error: any;
        params: Object;
    }>;
    cancel(isHook?: boolean): void;
    resolve({path, ctx}: {
        path: string;
        ctx?: Context;
    }): Promise<RouterResult>;
    run({path, ctx}: {
        path: string;
        ctx?: Context;
    }): Promise<RouterResult>;
}
